@@ -36,7 +36,7 @@ import IconButton from '../../components/IconButton';
 import LoadingCircle from '../../components/LoadingCircle';
 import LoadingDots from '../../components/LoadingDots';
 import { DEFAULT_MODEL } from '../../components/ModelSelector';
-import ModelSelector from "../../components/ModelSelector";
+import ModelSelector from '../../components/ModelSelector';
 import NextStepsPills from '../../components/NextStepsPills';
 import TextAndIconButton from '../../components/TextAndIconButton';
 import ToggleButton from '../../components/ToggleButton';
@@ -47,7 +47,7 @@ import MitoLogo from '../../icons/MitoLogo';
 import UndoIcon from '../../icons/UndoIcon';
 
 // Internal imports - Utils
-import { acceptAndRunCellUpdate, retryIfExecutionError } from '../../utils/agentActions';
+import { acceptAndRunCellUpdate, retryIfExecutionError, runAllCells } from '../../utils/agentActions';
 import { checkForBlacklistedWords } from '../../utils/blacklistedWords';
 import { createCheckpoint, restoreCheckpoint } from '../../utils/checkpoint';
 import { processChatHistoryForErrorGrouping, GroupedErrorMessages } from '../../utils/chatHistory';
@@ -101,6 +101,7 @@ import { codeDiffStripesExtension } from './CodeDiffDisplay';
 import { getFirstMessageFromCookie } from './FirstMessage';
 import ChatInput from './ChatMessage/ChatInput';
 import ChatMessage from './ChatMessage/ChatMessage';
+import RevertQuestionnaire from './ChatMessage/RevertQuestionnaire';
 import ScrollableSuggestions from './ChatMessage/ScrollableSuggestions';
 import { ChatHistoryManager, IDisplayOptimizedChatItem, PromptType } from './ChatHistoryManager';
 
@@ -108,6 +109,7 @@ import { ChatHistoryManager, IDisplayOptimizedChatItem, PromptType } from './Cha
 import '../../../style/button.css';
 import '../../../style/ChatTaskpane.css';
 import '../../../style/TextButton.css';
+import { getBase64EncodedCellOutput } from './utils';
 
 const AGENT_EXECUTION_DEPTH_LIMIT = 20
 
@@ -131,6 +133,7 @@ interface ICellStateBeforeDiff {
 }
 
 export type CodeReviewStatus = 'chatPreview' | 'codeCellPreview' | 'applied'
+export type AgentExecutionStatus = 'working' | 'stopping' | 'idle'
 
 const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     notebookTracker,
@@ -193,7 +196,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
         2. stopping: the agent is stopping after it has received ai response it is waiting on
         3. idle: the agent is idle
     */
-    const [agentExecutionStatus, setAgentExecutionStatus] = useState<'working' | 'stopping' | 'idle'>('idle')
+    const [agentExecutionStatus, setAgentExecutionStatus] = useState<AgentExecutionStatus>('idle')
 
     // We use a ref to always access the most up-to-date value during a function's execution. Refs immediately reflect changes, 
     // unlike state variables, which are captured at the beginning of a function and may not reflect updates made during execution.
@@ -210,6 +213,9 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     // Track if checkpoint exists for UI updates
     const [hasCheckpoint, setHasCheckpoint] = useState<boolean>(false);
 
+    // Track if revert questionnaire should be shown
+    const [showRevertQuestionnaire, setShowRevertQuestionnaire] = useState<boolean>(false);
+
     const updateModelOnBackend = async (model: string): Promise<void> => {
         try {
             await websocketClient.sendMessage({
@@ -247,6 +253,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     };
 
     const fetchChatHistoryAndSetActiveThread = async (threadId: string): Promise<void> => {
+
         const metadata: IFetchHistoryMetadata = {
             promptType: "fetch_history",
             thread_id: threadId
@@ -332,6 +339,9 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
         // Clear next steps when starting a new chat
         setNextSteps([])
 
+        // Get rid of the revert questionaire if its open
+        setShowRevertQuestionnaire(false);
+
         // Clear agent checkpoint when starting new chat
         setHasCheckpoint(false)
 
@@ -585,8 +595,8 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     const sendAgentExecutionMessage = async (
         input: string,
         messageIndex?: number,
-        sendActiveCellOutput: boolean = false,
-        selectedRules?: string[]
+        sendCellIDOutput: string | undefined = undefined,
+        additionalContext?: Array<{type: string, value: string}>
     ): Promise<void> => {
         // Step 0: reset the state for a new message
         resetForNewMessage()
@@ -599,17 +609,12 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
             newChatHistoryManager.dropMessagesStartingAtIndex(messageIndex)
         }
 
-        const agentExecutionMetadata = newChatHistoryManager.addAgentExecutionMessage(activeThreadIdRef.current, input, selectedRules)
+        const agentExecutionMetadata = newChatHistoryManager.addAgentExecutionMessage(activeThreadIdRef.current, input, additionalContext)
         if (messageIndex !== undefined) {
             agentExecutionMetadata.index = messageIndex
         }
 
-        if (sendActiveCellOutput) {
-            const activeCellOutput = await getActiveCellOutput(notebookTracker)
-            if (activeCellOutput !== undefined) {
-                agentExecutionMetadata.base64EncodedActiveCellOutput = activeCellOutput
-            }
-        }
+        agentExecutionMetadata.base64EncodedActiveCellOutput = await getBase64EncodedCellOutput(notebookTracker, sendCellIDOutput)
 
         setChatHistoryManager(newChatHistoryManager)
         setLoadingAIResponse(true);
@@ -627,7 +632,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     /* 
         Send whatever message is currently in the chat input
     */
-    const sendChatInputMessage = async (input: string, messageIndex?: number, selectedRules?: string[]): Promise<void> => {
+    const sendChatInputMessage = async (input: string, messageIndex?: number, additionalContext?: Array<{type: string, value: string}>): Promise<void> => {
         // Step 0: reset the state for a new message
         resetForNewMessage()
 
@@ -646,7 +651,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
             input,
             activeThreadIdRef.current,
             messageIndex,
-            selectedRules
+            additionalContext
         )
 
         setChatHistoryManager(newChatHistoryManager)
@@ -679,13 +684,14 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
     const handleUpdateMessage = async (
         messageIndex: number,
         newContent: string,
+        additionalContext?: Array<{type: string, value: string}>
     ): Promise<void> => {
 
         // Then send the new message to replace it
         if (agentModeEnabled) {
-            await startAgentExecution(newContent, messageIndex)
+            await startAgentExecution(newContent, messageIndex, additionalContext)
         } else {
-            await sendChatInputMessage(newContent, messageIndex)
+            await sendChatInputMessage(newContent, messageIndex, additionalContext)
         }
     };
 
@@ -888,7 +894,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
         setAgentExecutionStatus('idle');
     }
 
-    const startAgentExecution = async (input: string, messageIndex?: number, selectedRules?: string[]): Promise<void> => {
+    const startAgentExecution = async (input: string, messageIndex?: number, additionalContext?: Array<{type: string, value: string}>): Promise<void> => {
         await createCheckpoint(app, setHasCheckpoint);
         setAgentExecutionStatus('working')
 
@@ -900,7 +906,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
 
         let isAgentFinished = false
         let agentExecutionDepth = 1
-        let sendActiveCellOutput = false
+        let sendCellIDOutput: string | undefined = undefined
 
         // Loop through each message in the plan and send it to the AI
         while (!isAgentFinished && agentExecutionDepth <= AGENT_EXECUTION_DEPTH_LIMIT) {
@@ -913,12 +919,11 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
             // Only the first message sent to the Agent should contain the user's input.
             // All other messages only contain updated information about the state of the notebook.
             if (agentExecutionDepth === 1) {
-                await sendAgentExecutionMessage(input, messageIndex, false, selectedRules)
+                await sendAgentExecutionMessage(input, messageIndex, undefined, additionalContext)
             } else {
-                await sendAgentExecutionMessage('', undefined, sendActiveCellOutput)
-
+                await sendAgentExecutionMessage('', undefined, sendCellIDOutput)
                 // Reset flag back to false until the agent requests the active cell output again
-                sendActiveCellOutput = false
+                sendCellIDOutput = undefined
             }
 
             // Iterate the agent execution depth
@@ -1007,10 +1012,46 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                 }
             }
 
-            if (agentResponse.type === 'get_cell_output') {
-                // Mark that we should send the active cell output to the agent 
+            if (agentResponse.type === 'get_cell_output' && agentResponse.get_cell_output_cell_id !== null && agentResponse.get_cell_output_cell_id !== undefined) {
+                // Mark that we should send the cell output to the agent 
                 // in the next loop iteration
-                sendActiveCellOutput = true
+                sendCellIDOutput = agentResponse.get_cell_output_cell_id
+            }
+
+            if (agentResponse.type === 'run_all_cells') {
+                const result = await runAllCells(app, notebookTracker)
+                
+                // If run_all_cells resulted in an error, handle it through the error fixup process
+                if (!result.success && result.errorMessage && result.errorCellId) {
+                    // Set the error cell as active so the error retry logic can work with it
+                    setActiveCellByID(notebookTracker, result.errorCellId)
+                    
+                    const status = await retryIfExecutionError(
+                        notebookTracker,
+                        app,
+                        getDuplicateChatHistoryManager,
+                        addAIMessageFromResponseAndUpdateState,
+                        sendAgentSmartDebugMessage,
+                        previewAICodeToActiveCell,
+                        acceptAICode,
+                        shouldContinueAgentExecution,
+                        finalizeAgentStop,
+                        chatHistoryManagerRef
+                    )
+
+                    if (status === 'interupted') {
+                        break;
+                    }
+
+                    if (status === 'failure') {
+                        addAIMessageFromResponseAndUpdateState(
+                            "I apologize, but I encountered an error while running all cells and was unable to fix it after multiple attempts. You may want to check the notebook for errors.",
+                            'agent:execution',
+                            chatHistoryManager
+                        )
+                        break;
+                    }
+                }
             }
         }
 
@@ -1109,6 +1150,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
         */
         rejectAICode()
         setNextSteps([])
+        setShowRevertQuestionnaire(false);
     }
 
     const rejectAICode = (): void => {
@@ -1410,7 +1452,6 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                                 key={index}
                                 message={displayOptimizedChat.message}
                                 promptType={displayOptimizedChat.promptType}
-                                messageType={displayOptimizedChat.type}
                                 agentResponse={displayOptimizedChat.agentResponse}
                                 codeCellID={displayOptimizedChat.codeCellID}
                                 mitoAIConnectionError={displayOptimizedChat.type === 'connection error'}
@@ -1430,6 +1471,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                                 codeReviewStatus={codeReviewStatus}
                                 setNextSteps={setNextSteps}
                                 agentModeEnabled={agentModeEnabled}
+                                additionalContext={displayOptimizedChat.additionalContext}
                             />
                         )
                     }
@@ -1449,7 +1491,13 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                                 text="Revert changes"
                                 icon={UndoIcon}
                                 title="Revert changes"
-                                onClick={() => restoreCheckpoint(app, notebookTracker, setHasCheckpoint, getDuplicateChatHistoryManager, setChatHistoryManager)}
+                                onClick={() => {
+                                    void restoreCheckpoint(app, notebookTracker, setHasCheckpoint)
+                                    setDisplayedNextStepsIfAvailable(false)
+                                    setHasCheckpoint(false)
+                                    setShowRevertQuestionnaire(true)
+                                    scrollToDiv(chatMessagesRef);
+                                }}
                                 variant="gray"
                                 width="fit-contents"
                                 iconPosition="left"
@@ -1459,6 +1507,14 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                             </p>
                         </div>
                     )}
+                {/* Revert questionnaire - shows when user clicks revert button */}
+                {showRevertQuestionnaire && (
+                    <RevertQuestionnaire 
+                        onDestroy={() => setShowRevertQuestionnaire(false)} 
+                        getDuplicateChatHistoryManager={getDuplicateChatHistoryManager}
+                        setChatHistoryManager={setChatHistoryManager}
+                    />
+                )}
             </div>
             {displayOptimizedChatHistory.length === 0 && (
                 <div className="suggestions-container">
@@ -1483,6 +1539,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                     />
                 )}
                 <ChatInput
+                    app={app}
                     initialContent={''}
                     placeholder={
                         agentExecutionStatus === 'working' ? 'Agent is working...' :
@@ -1496,8 +1553,8 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                     isEditing={false}
                     contextManager={contextManager}
                     notebookTracker={notebookTracker}
-                    renderMimeRegistry={renderMimeRegistry}
                     agentModeEnabled={agentModeEnabled}
+                    agentExecutionStatus={agentExecutionStatus}
                 />
             </div>
             {agentExecutionStatus !== 'working' && agentExecutionStatus !== 'stopping' && (
@@ -1514,6 +1571,7 @@ const ChatTaskpane: React.FC<IChatTaskpaneProps> = ({
                                 setAgentModeEnabled(!isLeftSelected);
                                 // Clear agent checkpoint when switching modes
                                 setHasCheckpoint(false);
+                                setShowRevertQuestionnaire(false);
                                 // Focus the chat input directly
                                 const chatInput = document.querySelector('.chat-input') as HTMLTextAreaElement;
                                 if (chatInput) {
