@@ -18,4 +18,7 @@ export const COMMAND_MITO_AI_CELL_TOOLBAR_ACCEPT_CODE = `toolbar-button:accept-c
 export const COMMAND_MITO_AI_CELL_TOOLBAR_REJECT_CODE = `toolbar-button:reject-code`
 
 // Beta mode commands
-export const COMMAND_MITO_AI_BETA_MODE_ENABLED = `${MITO_AI}:beta-mode-enabled`
\ No newline at end of file
+export const COMMAND_MITO_AI_BETA_MODE_ENABLED = `${MITO_AI}:beta-mode-enabled`
+
+// Streamlit commands
+export const previewAsStreamlit = 'mito-ai:preview-as-streamlit';
