@@ -13,13 +13,17 @@ interface IconButtonProps {
     onClick: () => void;
     title: string;
     style?: React.CSSProperties;
+    notificationDotType?: 'success' | 'warning' | 'danger' | null;
+    className?: string;
+    disabled?: boolean;
 }
 
-const IconButton: React.FC<IconButtonProps> = ({ icon, onClick, title, style }) => {
+const IconButton: React.FC<IconButtonProps> = ({ icon, onClick, title, style, notificationDotType = null, className, disabled = false }) => {
 
     return (
-        <button className={classNames("icon-button")} onClick={onClick} title={title} style={style}>
+        <button className={classNames("icon-button", className)} onClick={onClick} title={title} style={style} disabled={disabled}>
             {icon}
+            {notificationDotType && <div className={`notification-dot notification-dot-${notificationDotType}`}></div>}
         </button>
     )
 }
